# -*- coding:utf-8 -*-
import re
import unicodedata
from urllib import quote, unquote
from urlparse import urlparse, urlunsplit, urlsplit


def to_unicode(content, from_charset=None):
    u"""Безопасное приведение к юникоду

    :type  content: str
    :param content: текст
    :type  from_charset: str
    :param from_charset: Кодировка исходного текста

    :rtype: unicode
    :returns: текст, преобразованный в юникод
    """
    if type(content) == unicode:
        return content

    charsets = {
        'utf-8' : 'utf8',
        'utf8' : 'utf8',
        'cp1251' : 'cp1251',
        'cp-1251' : 'cp1251',
        'windows-1251' : 'cp1251',
        'win-1251' : 'cp1251',
        '1251' : 'cp1251',
        'русdows-1251': 'cp1251',
        'koi8-r' : 'koi8-r'
    }
    if type(from_charset) in (str, unicode):
        from_charset = str(from_charset.lower())

    try:
        from_charset = charsets[from_charset]
        return unicode(content, encoding=from_charset)

    except KeyError:
        for from_charset in ('utf8', 'cp1251', 'koi8-r', None):
            try:
                if from_charset is not None:
                    return unicode(content, encoding=from_charset)
                else:
                    return unicode(content)
            except UnicodeError:
                continue

        raise UnicodeError('Can not be converted to Unicode')

    except UnicodeError:
        return unicode(content, encoding=from_charset, errors='ignore')
    
def get_full_domain_without_scheme(url):
    absolute_url = normalize(url).replace('//www.', '//')
    parsed = urlparse(absolute_url)
    return urlunsplit(('', parsed.netloc, '', '', '')).replace('//', '')
    
        
def normalize(url, charset='utf-8'):
    def _clean(string):
        string = unicode(unquote(string), 'utf-8', 'replace')
        return unicodedata.normalize('NFC', string).encode('utf-8')

    default_port = {
        'ftp': 21,
        'telnet': 23,
        'http': 80,
        'gopher': 70,
        'news': 119,
        'nntp': 119,
        'prospero': 191,
        'https': 443,
        'snews': 563,
        'snntp': 563,
    }

    if isinstance(url, unicode):
        url = url.encode(charset, 'ignore')

    if url[0] not in ['/', '-'] and ':' not in url[:7]:
        url = 'http://' + url

    url = url.replace('#!', '?_escaped_fragment_=')

    scheme, auth, path, query, fragment = urlsplit(url.strip())
    (userinfo, host, port) = re.search('([^@]*@)?([^:]*):?(.*)', auth).groups()

    scheme = scheme.lower()

    host = host.lower()
    if host and host[-1] == '.':
        host = host[:-1]
        # take care about IDN domains
    try:
        host = host.decode(charset).encode('idna')  # IDN -> ACE
    except Exception:
        host = host.decode(charset)

    path = quote(_clean(path), "~:/?#[]@!$&'()*+,;=")
    fragment = quote(_clean(fragment), "~")

    query = "&".join(["=".join([quote(_clean(t), "~:/?#[]@!$'()*+,;=") for t in q.split("=", 1)]) for q in query.split("&")])

    if scheme in ["", "http", "https", "ftp", "file"]:
        output = []
        for part in path.split('/'):
            if part == "":
                if not output:
                    output.append(part)
            elif part == ".":
                pass
            elif part == "..":
                if len(output) > 1:
                    output.pop()
            else:
                output.append(part)
        if part in ["", ".", ".."]:
            output.append("")
        path = '/'.join(output)

    if userinfo in ["@", ":@"]:
        userinfo = ""

    if path == "" and scheme in ["http", "https", "ftp", "file"]:
        path = "/"

    if port and scheme in default_port.keys():
        if port.isdigit():
            port = str(int(port))
            if int(port) == default_port[scheme]:
                port = ''

    auth = (userinfo or "") + host
    if port:
        auth += ":" + port
    if url.endswith("#") and query == "" and fragment == "":
        path += "#"
    return urlunsplit((scheme, auth, path, query, fragment))


# -*- coding:utf-8 -*-

import re

from yandex_yaca_parser.exceptions import YandexYacaParserError
from yandex_yaca_parser.utils import to_unicode, get_full_domain_without_scheme


class YandexYacaParser(object):
    params_regexr = re.U | re.M | re.DOTALL | re.I

    patterns = {
        'pagecount': re.compile(ur'<div class="info info_before-columns">Найдено сайтов:\s*([^<]+)\s*</div>', params_regexr),
        'title': re.compile(ur'<h2\s*class="yaca-snippet__title">\s*<a[^>]+href="([^"]+)"[^>]*>\s*(.*?)\s*</a>\s*</h2>', params_regexr),
        'descr': re.compile(ur'<div class="yaca-snippet__text">(.*?)</div>', params_regexr),
        'serp': re.compile(ur'<(li|div)\s*class="yaca-snippet">(.*?)</div>\s*</\1>', params_regexr),
        'category': re.compile(ur'<a[^>]+href="(/yaca/cat/[^"]+)"[^>]+>(.*?)</a>', params_regexr),
    }

    def __init__(self, content):
        self.content = to_unicode(content)

    def get_serp(self):
        pagecount = self.get_pagecount()
        snippets = self.get_snippets()
        return {'pc': pagecount, 'sn': snippets}

    def pagination_exists(self):
        return '<div class="pager"' in self.content

    def get_pagecount(self):
        patterns = (
            self.patterns['pagecount'],
        )

        for pattern in patterns:
            match = pattern.search(self.content)
            if match:
                break

        if not match:
            raise YandexYacaParserError(u'Не удалось найти кол-во сайтов')

        return int(re.sub('[^\d]', '', match.group(1)))

    def _get_title(self, sn):
        match = self.patterns['title'].search(sn)
        if not match:
            raise YandexYacaParserError(u'Не удалось найти заголовок')

        u = match.group(1).lower()
        t = match.group(2)
        try:
            d = get_full_domain_without_scheme(u)
        except UnicodeError as e:
            raise e
        return t, u, d

    def _get_descr(self, sn):
        match = self.patterns['descr'].search(sn)
        if not match:
            raise YandexYacaParserError(u'Не удалось найти описание')
        return match.group(1)

    def _get_category(self, sn):
        match = self.patterns['category'].search(sn)
        if match:
            return match.group(1).lower(), match.group(2)
        return '', ''

    def get_snippets(self):
        serp = self.patterns['serp'].findall(self.content)

        snippets = []
        for position, (_, sn) in enumerate(serp):
            t, u, d = self._get_title(sn)
            cu, c = self._get_category(sn)
            snippet = {
                'p': position + 1,
                't': t,
                'u': u,
                'd': d,
                's': self._get_descr(sn),
                'cu': cu,
                'c': c
            }
            snippets.append(snippet)
        return snippets



from setuptools import setup


VERSION = "0.0.3"

setup(
    name='yandex-yaca-parser',
    description="Parse yandex yaca. See https://yandex.ru/yaca/?text=test",
    version=VERSION,
    url='https://github.com/KokocGroup/yandex-yaca-parser',
    download_url='https://github.com/KokocGroup/yandex-yaca-parser/tarball/v{0}'.format(VERSION),
    packages=['yandex_yaca_parser'],
)
